#!/bin/bash

# 2014-04-23 booboo
# script to build html from all asciidoc files in asciidoc directory

#    Copyright (C) 2018 Bernd Stroessreuther <booboo@gluga.de>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# .-- Default values ------------------------------------------------------.

# Which browser do you want to use to browse your new website
# (if called with -b commandline parameter)
# you can give a full path or (if referenced in PATH) the name of the binary only
PREVIEW_BROSER="firefox"
#.

# .-- Defining global variables -------------------------------------------.

BINDIR=$(readlink -f $(dirname $0))
ADDIR=$(readlink -f "$BINDIR/../asciidoc")
HTMLDIR=$(readlink -f "$BINDIR/../htdocs")
RESOURCESDIR=$(readlink -f "$BINDIR/../web-resources")
TMPDIR=$(readlink -f "$BINDIR/../tmp")
#.

# .-- function: help -----------------------------------------------------.

function help {
    cat >&2 <<HelpEnd

    call using:
    $0 -h
        to display this help screen
    or

    $0 [-b] [-s <BASE_URL>]
        with
        -b      launch a browser directly after creating the HTML documents
                to preview them
        -s      create a sitemap.xml for Search Engine Optimization
                give the base URL of the Website as argument
                e. g.
                -s http://www.example.com

HelpEnd
}
#.

# .-- Commandline parameters ---------------------------------------------.

BROWSE=0
CREATE_SITEMAP=0
SITEMAP_BASE_URL=""

while getopts ":bs:h" opt; do
  case $opt in
    b)
      BROWSE=1
      ;;
    s)
      CREATE_SITEMAP=1
      SITEMAP_BASE_URL=$OPTARG
      ;;
    :)
      echo >&2
      echo "Option -$OPTARG requires an argument." >&2
      help
      exit 1
      ;;
    h)
      help
      exit 0
      ;;
    \?)
      echo >&2
      echo "Invalid option: -$OPTARG" >&2
      help
      exit 1
      ;;
  esac
done
#.

# .-- Checking / preparing the system ------------------------------------.
which asciidoc > /dev/null 2> /dev/null
RC=$?
if [[ $RC -ne 0 ]]; then
    echo
    echo "asciidoc not found, aborting $0"
    echo "please install asciidoc (e. g. from the repositories of your distribution)"
    echo "and make sure it can be found in your PATH"
    echo
    exit 1
fi

if [[ ! -d $HTMLDIR ]]; then
    mkdir -p $HTMLDIR || exit 2
fi

if [[ ! -d $TMPDIR ]]; then
    mkdir -p $TMPDIR || exit 3
fi
#.

# .-- function: sitemap --------------------------------------------------.

function sitemap {
    # call:
    #     sitemap init                 # to prepare the structure of the file
    #     sitemap add <URL> <MTIME>    # add at least one URL
    #     [sitemap add <URL> <MTIME>]  # add as many additional URLs as you like
    #     sitemap create               # to write the contents into the sitemap.xml file

    if [[ $CREATE_SITEMAP -eq 1 ]]; then
        local ACTION=$1
        local URL=$2
        local FILE_MTIME=$3

        case $ACTION in
        init)
            TMP_SITEMAP=$(mktemp --tmpdir=$TMPDIR)
            echo
            echo "### initializing new sitemap"
            echo "    in TMP file $TMP_SITEMAP"

            echo '<?xml version="1.0" encoding="UTF-8"?>' > $TMP_SITEMAP
            echo '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' >> $TMP_SITEMAP
            ;;
        add)
            echo "    <url>" >> $TMP_SITEMAP
            echo "        <loc>$URL</loc>" >> $TMP_SITEMAP
            echo "        <lastmod>$FILE_MTIME</lastmod>" >> $TMP_SITEMAP
            echo "    </url>" >> $TMP_SITEMAP
            ;;
        create)
            echo '</urlset>' >> $TMP_SITEMAP
            echo
            echo "### creating sitemap.xml"
            echo "    from TMP file $TMP_SITEMAP"
            mv $TMP_SITEMAP $HTMLDIR/sitemap.xml || rm $TMP_SITEMAP
            chmod 644 $HTMLDIR/sitemap.xml
            ;;
        *)
            echo "function sitemap called with unrecognized action $ACTION"
            ;;
        esac
    else
        if [[ -f $HTMLDIR/sitemap.xml ]]; then
            echo
            echo "### removing sitemap.xml"
            echo "    call with parameter -s if you want a sitemap.xml"
            rm -f $HTMLDIR/sitemap.xml
        fi
    fi
}
#.

sitemap init

echo
echo "### working in $HTMLDIR:"

for FILE in $(ls -1 $ADDIR/*.asciidoc)
do
    MTIME=$(stat -c %y $FILE | cut -d' ' -f1)
    BASENAME=$(basename $FILE .asciidoc)
    echo "   creating $BASENAME.html"
    asciidoc -b xhtml11 -o $HTMLDIR/$BASENAME.html $FILE
    sitemap add "$SITEMAP_BASE_URL/$BASENAME.html" $MTIME
done

sitemap create

if [[ -d $RESOURCESDIR ]]; then
    echo
    echo "### syncing static files"
    rsync -aP --delete --exclude *.html $RESOURCESDIR/* $HTMLDIR
else
    echo
    echo "### no static files found to sync"
    echo "maybe you want to create $RESOURCESDIR and put your images, stylesheets, ... there"
fi
echo

if [[ $BROWSE -eq 1 ]]; then
    $PREVIEW_BROSER $HTMLDIR/index.html
fi
